@@ -47,23 +47,41 @@ class TestCircular_hv:
             assert torch.all(
                 (hv == True) | (hv == False)
             ).item(), "values are either 1 or 0"
+        elif dtype in torch_complex_dtypes:
+            magnitudes = hv.abs()
+            assert torch.allclose(
+                magnitudes, torch.tensor(1.0, dtype=magnitudes.dtype)
+            ), "magnitude must be 1"
         else:
             assert torch.all(
                 (hv == -1) | (hv == 1)
             ).item(), "values are either -1 or +1"
 
         hv = functional.circular_hv(8, 1000000, generator=generator, dtype=dtype)
-        sims = functional.hamming_similarity(hv[0], hv).float() / 1000000
-        sims_diff = sims[:-1] - sims[1:]
+        if dtype in torch_complex_dtypes:
+            sims = functional.cosine_similarity(hv[0], hv)
+            sims_diff = sims[:-1] - sims[1:]
 
-        assert torch.all(
-            sims_diff.sign() == torch.tensor([1, 1, 1, 1, -1, -1, -1])
-        ), "second half must get more similar"
+            assert torch.all(
+                sims_diff.sign() == torch.tensor([1, 1, 1, 1, -1, -1, -1])
+            ), "second half must get more similar"
 
-        abs_sims_diff = sims_diff.abs()
-        assert torch.all(
-            (0.124 < abs_sims_diff) & (abs_sims_diff < 0.126)
-        ).item(), "similarity changes linearly"
+            abs_sims_diff = sims_diff.abs()
+            assert torch.all(
+                (0.248 < abs_sims_diff) & (abs_sims_diff < 0.252)
+            ).item(), "similarity changes linearly"
+        else:
+            sims = functional.hamming_similarity(hv[0], hv).float() / 1000000
+            sims_diff = sims[:-1] - sims[1:]
+
+            assert torch.all(
+                sims_diff.sign() == torch.tensor([1, 1, 1, 1, -1, -1, -1])
+            ), "second half must get more similar"
+
+            abs_sims_diff = sims_diff.abs()
+            assert torch.all(
+                (0.124 < abs_sims_diff) & (abs_sims_diff < 0.126)
+            ).item(), "similarity changes linearly"
 
     @pytest.mark.parametrize("sparsity", [0.0, 0.1, 0.756, 1.0])
     @pytest.mark.parametrize("dtype", torch_dtypes)
@@ -71,6 +89,10 @@ class TestCircular_hv:
         if not supported_dtype(dtype):
             return
 
+        if dtype in torch_complex_dtypes:
+            # Complex hypervectors don't support sparsity.
+            return
+
         generator = torch.Generator()
         generator.manual_seed(seed)
 
@@ -96,12 +118,6 @@ class TestCircular_hv:
 
     @pytest.mark.parametrize("dtype", torch_dtypes)
     def test_dtype(self, dtype):
-        if dtype in torch_complex_dtypes:
-            with pytest.raises(NotImplementedError):
-                functional.circular_hv(3, 26, dtype=dtype)
-
-            return
-
         if dtype == torch.uint8:
             with pytest.raises(ValueError):
                 functional.circular_hv(3, 26, dtype=dtype)
